@@ -5,11 +5,19 @@ from typing import Dict, List, Optional, Union, cast
 from cognite.client import CogniteClient
 from cognite.client.data_classes.assets import Asset, AssetList
 
-from . import replication
+from . import replication, datasets
 
 
 def build_asset_create(
-    src_asset: Asset, src_id_dst_map: Dict[int, int], project_src: str, runtime: int, depth: int
+    src_asset: Asset,
+    src_id_dst_map: Dict[int, int],
+    project_src: str,
+    runtime: int,
+    depth: int,
+    src_client: CogniteClient,
+    dst_client: CogniteClient,
+    src_dst_dataset_mapping: dict[int, int],
+    config: Dict,
 ) -> Asset:
     """
     Makes a new copy of the asset to be replicated based on the source asset.
@@ -20,6 +28,10 @@ def build_asset_create(
         project_src: The name of the project the object is being replicated from.
         runtime: The timestamp to be used in the new replicated metadata.
         depth: The depth of the asset within the asset hierarchy.
+        src_client: The client corresponding to the source project.
+        dst_client: The client corresponding to the destination project.
+        src_dst_dataset_mapping: Dictionary that maps source dataset ids to destination dataset ids
+        config: dict corresponding to the selected yaml config file
 
     Returns:
         The replicated asset to be created in the destination.
@@ -34,11 +46,23 @@ def build_asset_create(
         metadata=replication.new_metadata(src_asset, project_src, runtime),
         source=src_asset.source,
         parent_id=src_id_dst_map[src_asset.parent_id] if depth > 0 else None,
+        data_set_id=datasets.replicate(src_client, dst_client, src_asset.data_set_id, src_dst_dataset_mapping)
+        if config.get("dataset_support", False)
+        else None,
     )
 
 
 def build_asset_update(
-    src_asset: Asset, dst_asset: Asset, src_id_dst_map: Dict[int, int], project_src: str, runtime: int, depth: int
+    src_asset: Asset,
+    dst_asset: Asset,
+    src_id_dst_map: Dict[int, int],
+    project_src: str,
+    runtime: int,
+    depth: int,
+    src_client: CogniteClient,
+    dst_client: CogniteClient,
+    src_dst_dataset_mapping: dict[int, int],
+    config: Dict,
 ) -> Asset:
     """
     Makes an updated version of the destination asset based on the corresponding source asset.
@@ -51,6 +75,10 @@ def build_asset_update(
         runtime: The timestamp to be used in the new replicated metadata.
         depth: \\*\\*The depth of the asset within the asset hierarchy.
     \\*\\* only needed when hierarchy becomes mutable
+        src_client: The client corresponding to the source project.
+        dst_client: The client corresponding to the destination project.
+        src_dst_dataset_mapping: Dictionary that maps source dataset ids to destination dataset ids
+        config: dict corresponding to the selected yaml config file
 
     Returns:
         The updated asset object for the replication destination.
@@ -64,6 +92,11 @@ def build_asset_update(
     dst_asset.metadata = replication.new_metadata(src_asset, project_src, runtime)
     dst_asset.source = src_asset.source
     dst_asset.parent_id = src_id_dst_map[src_asset.parent_id] if depth > 0 else None  # when asset hierarchy is mutable
+    dst_asset.data_set_id = (
+        datasets.replicate(src_client, dst_client, src_asset.data_set_id, src_dst_dataset_mapping)
+        if config.get("dataset_support", False)
+        else None,
+    )
     return dst_asset
 
 
@@ -124,7 +157,10 @@ def create_hierarchy(
     dst_assets: List[Asset],
     project_src: str,
     runtime: int,
-    client: CogniteClient,
+    src_client: CogniteClient,
+    dst_client: CogniteClient,
+    src_dst_dataset_mapping: Dict[int, int],
+    config: Dict,
     subtree_ids: Optional[List[int]] = None,
     subtree_external_ids: Optional[List[str]] = None,
     subtree_max_depth: Optional[int] = None,
@@ -138,7 +174,10 @@ def create_hierarchy(
         dst_assets: A list of the assets that are in the destination.
         project_src: The name of the project the object is being replicated from.
         runtime: The timestamp to be used in the new replicated metadata.
-        client: The client corresponding to the destination project.
+        src_client: The client corresponding to the source project.
+        dst_client: The client corresponding to the destination project.
+        src_dst_dataset_mapping: dictionary mapping the source dataset ids to the destination ones
+        config: dict corresponding to the selected yaml config file
         subtree_ids: The id of the subtree root to replicate,
         subtree_external_ids: The external id of the subtree root to replicate,
         subtree_max_depth: The maximum tree depth to replicate,
@@ -162,6 +201,10 @@ def create_hierarchy(
             build_asset_update,
             project_src,
             runtime,
+            src_client,
+            dst_client,
+            src_dst_dataset_mapping,
+            config,
             depth=depth,
         )
 
@@ -169,12 +212,12 @@ def create_hierarchy(
         created_assets = replication.retry(
             create_assets_replicated_id_validation,
             create_assets,
-            function_create=client.assets.create,
-            function_list=client.assets.list,
+            function_create=dst_client.assets.create,
+            function_list=dst_client.assets.list,
         )
 
         logging.info(f"Attempting to update {len(update_assets)} assets.")
-        updated_assets = replication.retry(client.assets.update, update_assets)
+        updated_assets = replication.retry(dst_client.assets.update, update_assets)
 
         src_dst_ids = replication.existing_mapping(*created_assets, *updated_assets, *unchanged_assets, ids=src_dst_ids)
         logging.debug(f"Dictionary of current asset mappings: {src_dst_ids}")
@@ -224,6 +267,8 @@ def unlink_subtree_parents(
 def replicate(
     client_src: CogniteClient,
     client_dst: CogniteClient,
+    src_dst_datasets_mapping: Dict[int, int],
+    config: Dict,
     delete_replicated_if_not_in_src: bool = False,
     delete_not_replicated_in_dst: bool = False,
     subtree_ids: Optional[Union[int, List[int]]] = None,
@@ -236,6 +281,8 @@ def replicate(
     Args:
         client_src: The client corresponding to the source project.
         client_dst: The client corresponding to the destination project.
+        src_dst_datasets_mapping: dictionary mapping the source dataset ids to the destination ids.
+                config: dict corresponding to the selected yaml config file
         delete_replicated_if_not_in_src: If True, will delete replicated assets that are in the destination,
         but no longer in the source project (Default=False).
         delete_not_replicated_in_dst: If True, will delete assets from the destination if they were not replicated
@@ -279,7 +326,10 @@ def replicate(
         assets_dst,
         project_src,
         replicated_runtime,
+        client_src,
         client_dst,
+        src_dst_datasets_mapping,
+        config,
         subtree_ids,
         subtree_external_ids,
         subtree_max_depth,
